-- Process_AI_Mission_Results.lua
-- Author: sman
-- DateCreated: 01/11/18 4:06:05 AM
--------------------------------------------------------------

-- =================================================================================
--			Globals

gAiMissionTable = {}																		-- Master table holding all AI covert missions - both open and closed
g_Leader_Stats = {}																			-- Table holding verious information about the civ leaders in the game
g_Open_Mission_Limit = 3																	-- Initial value of the number of open missions a civ can have - should be adjusted by game difficulty later
g_Minimum_Turn_Interval = 10																-- Minimum number of turns between missions - should be adjusted by game difficulty later

COVERT_OPERATIONS_PROPENSITY = 1															-- Initial element ( [1] ) in g_Leader_Stats table; may want to add other elements later
	
-- =================================================================================
--			Functions

function PopulateLeaderStats()																-- Populate global table g_Leader_Stats at game start
	-- Add adjustments for leader stats: VictoryCompetitiveness, Boldness, Warmongerhate, Meanness; Major biases: MAJOR_CIV_APPROACH_WAR, MAJOR_CIV_APPROACH_AFRAID; Flavors: FLAVOR_ESPIONAGE, FLAVOR_OFFENSE, FLAVOR_DEFENSE
	g_Starting_Game_Turn = Game.GetGameTurn()												-- Saves the initial game turn - in case this is a loaded save game		

	for iPlayer=1, GameDefines.MAX_MAJOR_CIVS-1 do											-- Cycle through all major (non-human) players - everyone should be alive at this point (game start)		
		--if Players[iPlayer]:IsAlive() then													-- Only process living/playing civs
			local iLeaderType = Players[iPlayer]:GetLeaderType()
			local iCurrentEra = Players[iPlayer]:GetCurrentEra()
			local pLeaderInfo = GameInfo.Leaders[iLeaderType]
			local sLeaderName = GameInfo.Leaders[iLeaderType].Description or "unk"
			local iVicCompete = GameInfo.Leaders[iLeaderType].VictoryCompetitiveness or 3
			local iBoldness = GameInfo.Leaders[iLeaderType].Boldness or 3
			local iWarmongerHate = GameInfo.Leaders[iLeaderType].WarmongerHate or 3
			local iMeanness = GameInfo.Leaders[iLeaderType].Meanness or 3
			local iMajorWarApproach = 5
			local iMajorAfraidApproach = 5
			local iFlavorEspionage = 3
			local iFlavorOffense = 3
			local iFlavorDefense = 3

			for row in DB.Query("SELECT MajorCivApproachType, Bias FROM Leader_MajorCivApproachBiases WHERE LeaderType = '" .. pLeaderInfo.Type .."'" ) do	-- Get Major civ biases for this leader
				if row.MajorCivApproachType == "MAJOR_CIV_APPROACH_WAR" then
					 iMajorWarApproach = row.Bias
				elseif row.MajorCivApproachType == "MAJOR_CIV_APPROACH_AFRAID" then
					iMajorAfraidApproach = row.Bias
				end			
			end
			
			for row in DB.Query("SELECT FlavorType, Flavor FROM Leader_Flavors WHERE LeaderType = '" .. pLeaderInfo.Type .."'" ) do		-- Get appropriate Flavors for this leader
				if row.FlavorType == "FLAVOR_ESPIONAGE" then
					iFlavorEspionage = row.Flavor
				elseif row.FlavorType == "FLAVOR_OFFENSE" then
					iFlavorOffense = row.Flavor
				elseif row.FlavorType == "FLAVOR_DEFENSE" then
					iFlavorDefense = row.Flavor
				end			
			end					
		
			local iCOP = math.floor(  ((iVicCompete * 2) + (iBoldness*2) + (iWarmongerHate*1) + (iMeanness*2) + (iMajorWarApproach*2) + (iFlavorEspionage*6) + (iFlavorOffense*3) -
										(iMajorAfraidApproach*3) - (iFlavorDefense*1)) * .33  )	-- Adjust the COP value based on the weighting for each variable
										
			g_Leader_Stats[iPlayer] = { [COVERT_OPERATIONS_PROPENSITY] = iCOP }				-- Set the COP value into the leader table
						
			if g_Starting_Game_Turn > 10 then												-- Initial game turn is over 10; intent for loaded saved game - need to refresh the mission table
				Component1_Schedule_Mission(Players[iPlayer], g_Starting_Game_Turn, iCurrentEra)		-- Determine if iPlayer can schedule an attack planned for this game turn
			end
		--end
	end

	-- ========================================================================================================================
	-- Misc. adjustments, based on game difficulty, etc.					
	local iGameSpeed = Game.GetGameSpeedType()												-- 0= marathon, 1=epic, 2=std, 3=quick
	local iHandicap = Game:GetHandicapType()												-- 0= settler, 1=chieftan, 2=warlord, 3=prince, 4=king, 5=emp, 6=immortal, 7 = deity
		
	g_Open_Mission_Limit = (4 - iGameSpeed) + 1												-- AI can have 2 missions open on quick, 3 on std, 4 on epic, and 5 on marathon 
	if g_Open_Mission_Limit > 5 then g_Open_Mission_Limit = 5 end							-- Add upper bound in case another mod adds a new gamespeed
	if g_Open_Mission_Limit < 1 then g_Open_Mission_Limit = 1 end							-- Add lower bound in case another mod adds a new gamespeed

	g_Minimum_Turn_Interval = (8 - iHandicap) * 2											-- AI minimum launch window of "2" turns on deity, 8 turns on king, and 16 turns on settler
	if g_Minimum_Turn_Interval > 25 then g_Minimum_Turn_Interval = 25 end					-- Add upper bound in case another mod adds a new difficulty level
	if g_Minimum_Turn_Interval < 3 then g_Minimum_Turn_Interval = 3 end						-- Add lower bound in case another mod adds a new difficulty level (for immortal, deity)

	--print("Open Mission Limit: " .. g_Open_Mission_Limit .. "    Min Turn Interval: " .. g_Minimum_Turn_Interval .. "     Initial Game Turn: " .. g_Starting_Game_Turn)

end
Events.SequenceGameInitComplete.Add( PopulateLeaderStats )


function OpenMissionStats (pPlayer)															-- Determine how many open missions pPlayer has, and what game turn the most recent mission is supposed to occur
	local iMissionCount = 0
	local iLatestMissionTurn = -1
	local pPlayerID = pPlayer:GetID()	
	if #gAiMissionTable > 0 then
		for iIndex = 1, #gAiMissionTable do
			if (gAiMissionTable[iIndex][1] == pPlayerID) and (gAiMissionTable[iIndex][2] == YES) then		-- Does this mission belong to pPlayerID and is it still open?
				iMissionCount = iMissionCount + 1
				if gAiMissionTable[iIndex][5] > iLatestMissionTurn then
					iLatestMissionTurn = gAiMissionTable[iIndex][5]
				end
			end
		end
	end
	--print ("Total missions: " .. #gAiMissionTable .. "    total missions for player " .. pPlayerID .. ":  " .. iMissionCount)
	return iMissionCount, iLatestMissionTurn
end


function GetTargetPlayer(pPlayer)															-- Select a target civ for this operation; return -1 if no valid targets
	local iTargetPlayerID = -1																-- Evaluates a set of fixed criteria to determine the relationship between the attacking civ and all other civs (human too)
	local pPlayerTeam = Teams[pPlayer:GetTeam()]											-- Then adjusts those criteria by a random +- 25%.  Civ with the "worst" relationship is the target
	local iWorstEnemyCount = 0	
	--print("Getting a target player for player: " .. pPlayer:GetID())
	if math.random(1, 100) < 33 then														-- Get a truly random civ to attack (33% of the time)
		local tLivingCivs = {}
		for iPlayer=0, GameDefines.MAX_MAJOR_CIVS-1 do										-- Cycle through all major players - everyone should be alive at this point (game start)		
			local pTempTeam = Teams[Players[iPlayer]:GetTeam()]								-- Get team pointer for loop player

			if pTempTeam:IsHasMet(pPlayer:GetTeam()) and									-- Only consider civs we've met
					(Players[iPlayer]:IsAlive()) and										-- Only consider living teams
					(iPlayer ~= pPlayer:GetID()) then										-- Only consider civs who are not ourselves			
				tLivingCivs[#tLivingCivs] = Players[iPlayer]:GetID()						-- Add this player ID to the table of possible targets
			end
		end
		if #tLivingCivs > 0 then															-- If there are 1 or more potentil targets, then
			iTargetPlayerID = Players[math.random(1, #iWorstEnemyID)]:GetID()					-- Worst enemy is an equally random living major civ
		else																				-- Any issues means the function will return -1 and no enemy is selected
			iTargetPlayerID = -1
		end
	else																					-- Use a formula/process to determine worst enemy
		for iPlayer=0, GameDefines.MAX_MAJOR_CIVS-1 do										-- Cycle through all major players - everyone should be alive at this point (game start)		
			local pTempTeam = Teams[Players[iPlayer]:GetTeam()]								-- Get team pointer for loop player
			if pTempTeam:IsHasMet(pPlayer:GetTeam()) and									-- Only consider civs we've met
					(Players[iPlayer]:IsAlive()) and										-- Only consider living teams
					(iPlayer ~= pPlayer:GetID()) then										-- Only consider civs who are not ourselves				
				local iBadnessCount = 0														-- Badness is a summary of all the things the target civ has done against pPlayer
				local iEnemyTeam = Players[iPlayer]:GetTeam()								-- Some methods need target team ID instead of target player ID

				if pPlayer:IsCapitalCapturedBy(Players[iPlayer]:GetID()) then				-- They have our capital...

					iBadnessCount = iBadnessCount + 20
				end

				if (pPlayer:IsNukedBy(Players[iPlayer]:GetID())) then						-- The nuked us in the past...
					iBadnessCount = iBadnessCount + 10
				end

				if pPlayerTeam:IsAtWar(iEnemyTeam) then										-- We are at war with them...
					iBadnessCount = iBadnessCount + 10
				end

				if Players[iPlayer]:IsDenouncingPlayer(pPlayer:GetID()) then				-- They are denouncing us...
					iBadnessCount = iBadnessCount + 7
				end

				if pPlayer:IsPlayerRecklessExpander(Players[iPlayer]:GetID()) then			-- They are reckless expanders...
					iBadnessCount = iBadnessCount + 5
				end

				if Players[iPlayer]:IsHasPlayerBulliedProtectedMinor() then					-- They have bullied protected city states...
					iBadnessCount = iBadnessCount + 2
				end

				if Players[iPlayer]:IsEmpireUnhappy() then									-- They are unhappy...
					iBadnessCount = iBadnessCount + 1
				end
				if Players[iPlayer]:IsEmpireVeryUnhappy() then								-- They are very unhappy...
					iBadnessCount = iBadnessCount + 2
				end
				if Players[iPlayer]:IsEmpireSuperUnhappy() then								-- They are super unhappy...
					iBadnessCount = iBadnessCount + 4
				end
			
				local iPlayerPower = pPlayer:GetPower()										-- Evaluate relative strength.  Get pPlayer's "power"
				local iTargetPlayerPower = Players[iPlayer]:GetPower()						-- Get Target player's "power"
				local iPowerRatio = math.floor(iTargetPlayerPower / iPlayerPower)			-- Get the ratio of strength
				if iPowerRatio < 1 then iPowerRatio = math.floor(iPlayerPower / iTargetPlayerPower) * -1 end	-- If enemy is stronger, then reverse the ration and make it negative
				if iPowerRatio > 3 then iPowerRatio = 3 end									-- After 3 times larger, it doesn't matter
				if iPowerRatio < -3 then iPowerRatio = -3 end								-- After 3 times smaller, it doesn't matter

				iBadnessCount = iBadnessCount + (iPowerRatio * 4)							-- Change iBadnessCount: if we are stronger, add the Ratio*4; if they are stronger, subtract the reversered ration by Ratio*4; max is +-12 points

				local iAdjustment = (math.random(1, 50) - 25) + 100							-- Add some randomness to the overall equation (+- 25%)			
				--print("Target civ: " .. iPlayer .. "   ibad: " .. iBadnessCount .. "    Adjustment: " .. iAdjustment)			
				iBadnessCount = math.floor (iBadnessCount * (iAdjustment/100) )				-- Adjust the overall badness 			

				if iBadnessCount > iWorstEnemyCount then
					iTargetPlayerID = Players[iPlayer]:GetID()	
					iWorstEnemyCount = iBadnessCount
				end
			end
		end
	end
	--print("Worst enemy/target: " .. iTargetPlayerID)
	return iTargetPlayerID																	-- Returns -1 if no civs can be attacked - not sure how this could happen, though
end


function GetMissionNumber(pPlayer, iCurrentEra)												-- Select a mission to attempt; return -1 if no valid mission exists
	-- Level 1 missions occur 35%, L2=30%, L3=25%, L4=10%
	-- Might add in change to vary mission selection based on leader stats: i.e. greedy types like gold; meanness=poison/plague; offensive types=partisans/steal city, etc.
	local iMissionNumber = -1																-- Default return value	
	local iUpperBound = 35																	-- Upper bound limit for L1 missions
	if (iCurrentEra==2) or (iCurrentEra==3) then iUpperBound = 65							-- Upper bound limit for L2 missions
	elseif (iCurrentEra==4) or (iCurrentEra==5) then iUpperBound = 90						-- Upper bound limit for L3 missions
	elseif (iCurrentEra==6) or (iCurrentEra==7) then iUpperBound = 100 end					-- Upper bound limit for L4 missions	
	local iMissionPercent = math.random(1, iUpperBound)										-- Limit mission possibilities based on Era of attacking AI player
	--print("GetMissionNumber. Era: " .. iCurrentEra .. "    UpperBound: " .. iUpperBound .. "    iMissionPercent: " .. iMissionPercent)
	if iMissionPercent < 8 then																-- L1: Poison water (7%)
		iMissionNumber = 2
	elseif iMissionPercent < 26 then														-- L1: Spread rumors (18%)  - if iMissionPercent < 20, will have already trapped before reaching here; this block executes if iMissionPercent is 20..29
		iMissionNumber = 3
	elseif iMissionPercent < 36 then														-- L1: Pillage improvements (10%)
		iMissionNumber = 4
	
	elseif iMissionPercent < 48 then														-- L2: Sabotage production (12%)
		iMissionNumber = 5
	elseif iMissionPercent < 59 then														-- L2: Steal gold (11%)
		iMissionNumber = 6 
	elseif iMissionPercent < 66 then														-- L2: Spread plague (7%)
		iMissionNumber = 7
	
	elseif iMissionPercent < 76 then														-- L3: Destroy building (10%)
		iMissionNumber = 8
	elseif iMissionPercent < 81 then														-- L3: Recruit partisans (5%)
		iMissionNumber = 9
	elseif iMissionPercent < 91 then														-- L3: Spread propaganda (10%)
		iMissionNumber = 10
	
	elseif iMissionPercent < 94 then														-- L4: Cause resistance (4%)
		iMissionNumber = 11
	elseif iMissionPercent < 98 then														-- L4: Cause anarchy (4%)
		iMissionNumber = 12
	elseif iMissionPercent < 101 then														-- L4: Steal city (2%)
		iMissionNumber = 13
	else
		print("GetMissionNumber - cannot assign a mission.  iMissionPercent: " .. iMissionPercent)
	end		
	return iMissionNumber																	-- Returns -1 if no valid mission assigned
end


function BuildWarningString(pPlayer, iGameTurn, iIndex, iEspionagePosture, bTrueWarning)	-- Builds a warning message for Notifications; adds ambiguity to a report based on pPlayers Espionage Posture
																							-- Last argument="Is this a warning" if true, then future/impending report; if false then past reporting									
	local sWarnString = ""																	-- Return warning message value	
	local sFadeString = ""																	-- Return fade value
	
	local sRumorReport = "Operation results unknown."										-- What is the default report value?
	local sRumorReporter = "unknown"														-- Who is issuing the report?
	local sRumorAttacker = "unknown"														-- Who is attacking in the report?
	local sRumorLocation = "a distant land"													-- Where did they attack?  City?  Civ?	
	local sRumorTurn = "recently"															-- Game turn mission executed	
	local iReportQuality = 0
	
	local sActualReporter = Players[gAiMissionTable[iIndex][7]]:GetCivilizationAdjective()	-- Adjective of actual target civ;  [7] = iTargetPlayerID
	local sActualTargetCiv = Players[gAiMissionTable[iIndex][7]]:GetCivilizationShortDescription()	-- Name of actual target civ;  [7] = iTargetPlayerID
	local sActualTargetCity = gAiMissionTable[iIndex][8]:GetName()							-- Name of actual target city;  [8] = target city pointer
	local sActualAttacker = Players[gAiMissionTable[iIndex][1]]:GetCivilizationAdjective()	-- Adjective of actual target civ;  [1] = mission owner PlayerID
	
	local pCapCity = pPlayer:GetCapitalCity()												-- Get the capital city for pPlayer 	
	local iX = pCapCity:GetX()																-- Default the notification X to the pPlayer capital city X
	local iY = pCapCity:GetY()																-- Default the notification Y to the capital city Y	
					
	local iReportStrenth = iEspionagePosture + (math.random(10, 30) - 15)					-- Strength of pPlayer Espionage Posture +- 15%
	if iReportStrenth < 10 then iReportStrenth = 10	end										-- Minimum chance of getting good report is 10%
	if bTrueWarning then																	-- For "true warnings" (attacks against the human player) the upper confidence bound is higher
		if iReportStrenth > 80 then iReportStrenth = 80 end									-- Maximum chance of getting good report is 80% for true warnings
	else																					-- For not "true warnings" the confidence limit is lower
		if iReportStrenth > 50 then iReportStrenth = 50 end									-- Maximum chance of getting good report is 50% for normal reports
	end

	-- Determine the specificity of the report, based on the iReportStrength
	
	if math.random(1, 100) < iReportStrenth then											-- Chance of WHO reported; defaults to "unk"
		sRumorReporter = sActualReporter													-- Change WHO reported to the actual target civ
		iReportQuality = iReportQuality + 1													-- Increase report quality
	end

	if math.random(1, 100) < iReportStrenth then											-- Chance of WHO attacked; defaults to "unk"
		sRumorAttacker = sActualAttacker													-- Change WHO attacked to the actual attacking civ
		iReportQuality = iReportQuality + 1													-- Increase report quality
	end

	if math.random(1, 100) < iReportStrenth then											-- Chance of WHERE attacked; defaults to "distand land"
		if math.random(1, 100) < iReportStrenth  then										-- Determine if reporting City or Civ
			sRumorLocation = sActualTargetCity												-- Return the city
			if (gAiMissionTable[iIndex][8]) then											-- If the target city exists (i.e. not captured, razed, etc.), then; only chance Notification location if we actually know the city
				iX = gAiMissionTable[iIndex][8]:GetX()										-- Set the notification X to the target city X
				iY = gAiMissionTable[iIndex][8]:GetY()										-- Set the notification Y to the target city Y
			end
		else
			sRumorLocation = sActualTargetCiv												-- Return the civ
		end
		if not sRumorLocation then sRumorLocation = sActualTargetCiv end					-- If for some reason the location is nil, return the civ		
		iReportQuality = iReportQuality + 1													-- Increase report quality
	end

	if math.random(1, 100) < iReportStrenth then											-- Chance of actual report; defaults to "Results unk"
		sRumorReport = gAiMissionTable[iIndex][9]											-- Change report to the actual report
		iReportQuality = iReportQuality + 1													-- Increase report quality
	end

	if math.random(1, 100) < iReportStrenth then											-- Chance of actual mission turn; defaults to current game turn
		local iAdjustTurn = math.random(1, 3) - 2											-- Adjust the turn +- 1
		sRumorTurn = "on game turn " .. gAiMissionTable[iIndex][5] + iAdjustTurn			-- Change to the actual mission turn		
		iReportQuality = iReportQuality + 1													-- Increase report quality
	end
	
	if bTrueWarning then																	-- Adjust the warning ambiguity to more details that a home-civ should have	
		local sMissionType = "unknown"														-- Default mission type to report
		local iMissionChances = math.random(1, 100)											-- Chance of discovering exactly what the enemy is up to
		if iMissionChances < math.floor(iReportStrenth / 2) then							-- The iMissionsChance is less than half of the successful report number
			sMissionType = "'" .. g_MissionList[gAiMissionTable[iIndex][6]][1] .. "'"		-- Get the exact name of impending attack
		elseif 	iMissionChances < iReportStrenth then
			sMissionType = "Level " .. g_MissionList[gAiMissionTable[iIndex][6]][6]			-- Returns the level of the planned attack
		end
		
		if sRumorLocation == "a distant land" then sRumorLocation = sActualTargetCiv end	-- Ensure the target location is either a city or civ; no "unknowns" allowed
		if sRumorTurn == "recently"	then sRumorTurn = "soon" end							-- If we don't have specific game turn, change default to a future time
		
		sWarnString = "Your CI services report " .. sRumorAttacker .. " agents are planning to conduct a " .. sMissionType .. " Covert Operation against " .. sRumorLocation .. " " .. sRumorTurn .. "."
		sFadeString = "RUMINT: your agents warn of an impending Covert Operation against " .. sRumorLocation
	else
		sWarnString ="Reports by " .. sRumorReporter .. " sources state that " .. sRumorTurn .. ", " .. sRumorAttacker .. " foreign agents conducted a Covert Operation in " .. sRumorLocation .. "."
		sWarnString = sWarnString .. " " .. sRumorReport
		sFadeString = "RUMINT: " .. sRumorAttacker .. " agents conduct Covert Operation in " .. sRumorLocation
	end
	return sWarnString, sFadeString, iX, iY, iReportQuality									-- Report quality is the # of specific items in the report (max = 5)
end



function DisplayAIMissionTable(iGameTurn)													-- Display the contents of the AI Mission Table
	print("==============================================================================================================")
	print("Mission Table Contents.  Game Turn: " .. iGameTurn)
	if #gAiMissionTable == 0 then
		print("AI Mission Table is currently empty.")
	else
		for iIndex = 1, #gAiMissionTable do
			local sMissionStatus = "Open"
			if gAiMissionTable[iIndex][2] == NO then sMissionStatus = "Closed" end
			local sTempCityName = gAiMissionTable[iIndex][8]:GetName()
			if not sTempCityName then sTempCityName = "no longer valid" end
			print("Index: " .. iIndex .. "    Player: " .. gAiMissionTable[iIndex][1] .. "    Mission Status: " .. sMissionStatus .. "  Scheduled Game Turn: " .. gAiMissionTable[iIndex][3] .. "  Launch Game Turn: " .. gAiMissionTable[iIndex][5] .. "    Target Civ: " .. gAiMissionTable[iIndex][7] .. "    Target City: " .. sTempCityName .. "    Mission Number: " .. gAiMissionTable[iIndex][6] .. "    Results: " .. gAiMissionTable[iIndex][9])
		end
	end
	print("==============================================================================================================")	
end


function Component1_Schedule_Mission(pPlayer, iGameTurn, iCurrentEra)						-- Determines if pPlayer can schedule a mission this turn
	local iOpenMissionCount, iLatestMissionTurn = OpenMissionStats (pPlayer)			-- Get # of open missions and latest mission date for this player
		
	if iOpenMissionCount >= (iCurrentEra + 1) then return end							-- Can never have more open missions than the Era+1 you're in
	if iOpenMissionCount >= g_Open_Mission_Limit then return end						-- Only let pPlayer have g_Open_Mission_Limit open missions at a time, which is adjusted for difficulty/gamespeed
	if (iGameTurn > 20) and (iLatestMissionTurn > (iGameTurn - g_Minimum_Turn_Interval)) then return end		-- Force a g_Minimum_Turn_Interval or more game turn wait between missions, adjusted for difficulty/gamespeed
				
	local iToAttackOrNotToAttack = math.random(1, 100)									-- Chance of scheduling an Attack
	if iToAttackOrNotToAttack > g_Leader_Stats[pPlayer:GetID()][COVERT_OPERATIONS_PROPENSITY] then return end		-- If random % is greater than COP threashold, exit early; otherwise SCHEDULE!
			
	local iTargetPlayerID = GetTargetPlayer(pPlayer)									-- Determine Target Civ - Who do we hate the most?
	if iTargetPlayerID == -1 then return end											-- If no valid civs to attack, exit early
		
	local iMissionNumber = GetMissionNumber(pPlayer, iCurrentEra)						-- Determine Mission to launch
	if iMissionNumber == -1 then return end												-- If no valid mission to launch, exit early
		
	local iTurnToExecute = iGameTurn + math.random(8, 12)								-- Determine the actual turn to launch the mission			
	local iTurnToWarn = iTurnToExecute - math.random(0, 5)								-- Higher bound here needs to be 1 less than the lower bound of iTurnToExecute!
	if iTurnToWarn == iTurnToExecute then iTurnToWarn = -1 end							-- Sometimes, you don't get a warning...
		
	local iIndex = #gAiMissionTable + 1													-- New row index in the Mission_Table
	if iIndex == 0 then iIndex = 1 end													-- Ensure table index starts no lower than 1
		
	local bIsCapitalOK = true															-- Default to let capitals be valid targets for covert ops
	if iMissionNumber == STEAL_CITY then bIsCapitalOK = false end						-- If the mission is Steal City, then the capital is not a valid target
	local pTargetCity = GetRandomCity(Players[iTargetPlayerID], bIsCapitalOK)			-- Arguments: pPlayer, bIsCapitalOK
		
	if not pTargetCity then return end													-- If nil, no available cities (e.g. steal city and target civ has only capital?) - return early
			
	-- gAiMissionTable[iIndex][1] = attacking player ID    [2] = Is the mission open?  Will change to NO once the mission is completed    [3] = What turn was mission scheduled?
	--		[4] = What turn to warn?    [5] = What turn to execute?   [6] = What mission type will we launch?    [7] = Who is the target player(ID)?    
	--		[8] = Target City pointer   [9] = Placeholder element - used to store mission results after the mission - for future reference
	gAiMissionTable[iIndex] = { [1] = pPlayer:GetID(), [2] = YES, [3] = iGameTurn, [4] = iTurnToWarn, [5] = iTurnToExecute, [6] = iMissionNumber, [7] = iTargetPlayerID, [8] = pTargetCity, [9] = ""}

	print("Scheduling a mission. Index: " .. iIndex .. "    Player: " .. pPlayer:GetID() .. "  on game turn: " .. iGameTurn .. "    warning turn: " .. iTurnToWarn .. "  planned turn: " .. iTurnToExecute .. "    target civ: " .. iTargetPlayerID ..  "    target city: " .. pTargetCity:GetName() .. "    mission number: " .. iMissionNumber)

end


function Component2_Warn_Mission(pPlayer, iGameTurn)										-- Determines if a Warning Notification is required this turn - only for human player!
	-- There is a small chance of receiving a report about covert operation directed against another civ.  Chance is based on the civ's EspionagePosture: capital's buildinginfluence (0..68) and #spies*2 - may develop this further
	local iEspionagePosture = CalculateEspionagePosture(pPlayer)
	local sWarnString = ""
	local sFadeString = ""
	local iReportQuality = 0
	local iX = -1
	local iY = -1
	
	local iChanceToView = math.floor(iEspionagePosture / 4)									-- EP / 4 = chance to view foreign report
	if iChanceToView < 2 then iChanceToView = 2												-- Minimum chance to view is 2%
	elseif iChanceToView > 10 then iChanceToView = 10 end									-- Maximum chance to view is 10%

	if math.random(1, 100) < iChanceToView then												-- Can we see the report?
		for iIndex = #gAiMissionTable, 1, -1 do												-- Cycle through all missions in the table backwards - looking for first closed mission that isn't directed at pPlayer
			if (gAiMissionTable[iIndex][2] == NO) and (gAiMissionTable[iIndex][7] ~= pPlayer:GetID()) then	-- Is the mission CLOSED?  Is the target NOT a human?  If so, grab the most recent report										
				sWarnString, sFadeString, iX, iY, iReportQuality = BuildWarningString(pPlayer, iGameTurn, iIndex, iEspionagePosture, false)	-- last argument="Warning" if true, then future/impending report; if false then past reporting
				if iReportQuality >= 2 then													-- Message has to have 2 or more elements "correct" otherwise, abandon
					SendNotification(4, sWarnString, sFadeString, nil, iX, iY)				-- Issue Notification
					break																	-- Exit the loop early
				end				
			end
		end																					-- If the loop terminates normally, we didn't find any CLOSED missions targeting non-human players
	end																						-- If no missions found, no notification sent, continue on with processing...
	
	for iIndex = 1, #gAiMissionTable do														-- Cycle through all missions in the table		
		if (gAiMissionTable[iIndex][2] == YES) and (gAiMissionTable[iIndex][7] == pPlayer:GetID()) and (gAiMissionTable[iIndex][4] == iGameTurn) then	-- Open mission, target human, warn turn = game turn					
			sWarnString, sFadeString, iX, iY, iReportQuality = BuildWarningString(pPlayer, iGameTurn, iIndex, iEspionagePosture, true)	-- last argument="Warning" if true, then future/impending report; if false then past reporting									
			SendNotification(4, sWarnString, sFadeString, nil, iX, iY)						-- Issue Notification
		end
	end
end

function Component3_Execute_Mission(pPlayer, iGameTurn, iCurrentEra)						-- Actually executes a previously-scheduled mission
	local pUnit = nil																		-- AI doesn't use units to launch Covert Ops
		
	for iIndex = 1, #gAiMissionTable do													-- Cycle through all missions

		if (gAiMissionTable[iIndex][1] == pPlayer:GetID()) and (gAiMissionTable[iIndex][5] == iGameTurn) and (gAiMissionTable[iIndex][2] == YES) then		-- pPlayer:GetID() has a mission scheduled for this turn, and if the mission is OPEN?

			-- Mission results:		    Type	FadeStr	Mission Result	Reaction	OP Fate		X		Y		Msn		Survive		Incident	Summary  -- default values for mission results data table
			--							1		2		3				4			5			6		7		8		9			10			11
			local tMissionResultData = { 1,		"",		"",				"",			"",			-1,		-1,		1,		1,			1,			"" }		-- Set up results table for passing around	
			local pTargetPlayer = Players[gAiMissionTable[iIndex][7]]						-- Set the target player pointer
			local iMissionNumber = gAiMissionTable[iIndex][6]								-- Set the mission number
			local pTargetCity = gAiMissionTable[iIndex][8]									-- Get the target city pointer
			
			if pTargetCity then																-- If pTargetCity is valid, then									
				GetMissionResult(tMissionResultData, iMissionNumber, pPlayer, pUnit, iCurrentEra, pTargetPlayer, pTargetCity)	-- Get results of AI CO mission
				GetInternationalResponses(tMissionResultData, iMissionNumber, pPlayer, pUnit, pTargetPlayer, pTargetCity)		-- Get international reaction to AI CO mission		
				
				if pTargetPlayer:IsHuman() then												-- Only build/display Notification if the human player is the AI CO target
					local sAttackerInfo = "an unidentified civilization"					-- Default attacker ID to report
					local sAppearenceInfo = "appear to be"									-- Default "certainty"
					local iEspionagePosture = CalculateEspionagePosture(pPlayer) or 0		-- Target civ's EP (0..68 + (# Spies * 2)
					local iBuildingInfluence = GetBuildingInfluence(pTargetCity) or 0		-- Target City defensive buildings (0..68)
					local iUnitInfluence = GetUnitInfluence(pTargetCity) or 0				-- # units*1 and Level of Covert Op units * 3

					iEspionagePosture =  math.floor(iEspionagePosture / (pPlayer:GetNumCities() + 1))	-- Determine how "thin" the resources are stretched; no div by 0 here...
					local iChanceToIDAttacker = math.floor(iEspionagePosture + (iBuildingInfluence / 3) + iUnitInfluence)	-- Chance to ID attacker
					
					if math.random(1, 100) < iChanceToIDAttacker then						-- Is the random # less than the chance to ID threshold?  Then,
						sAttackerInfo = pPlayer:GetCivilizationDescription()				-- Set the ID string from unid'd to id'd
						if math.random(1, 100) < iChanceToIDAttacker then sAppearenceInfo = "have been identified, coming" end	-- Check to see if we're certain or guessing
					end
										
					tMissionResultData[5] = "The perpetrators " .. sAppearenceInfo .. " from " .. sAttackerInfo		-- Set the OP Fate element to include in the Notification
										
					local sSeparatorString = "[NEWLINE][NEWLINE]"									-- Experiment with how to separate message components
					--local sSeparatorString = "  "
					local sLongMessage = tMissionResultData[3]										-- Build the long message string piece by piece
					if tMissionResultData[4] ~= "" then sLongMessage = sLongMessage .. sSeparatorString .. tMissionResultData[4] end
					if tMissionResultData[5] ~= "" then sLongMessage = sLongMessage .. sSeparatorString .. tMissionResultData[5] end
					SendNotification(tMissionResultData[1], sLongMessage, tMissionResultData[2], pUnit, tMissionResultData[6], tMissionResultData[7])	-- Issue Notification
				end	
							
				--gAiMissionTable[iIndex][9] = (tMissionResultData[8]*100) + (tMissionResultData[10]*10) + tMissionResultData[9]		-- Add the results, 1=Yes, 0=No; 101= Yes No Yes			
				gAiMissionTable[iIndex][9] = tMissionResultData[2] or "Status unknown"		-- Use the fade string as the summary message
			else																			-- pTargetCity no longer valid - city doesn't exist or is captured (therefore new pointer)			
				gAiMissionTable[iIndex][9] = "Status unknown"								-- Use the fade string as the summary message; may need to look here further
			end

			gAiMissionTable[iIndex][2] = NO													-- Show the mission as completed
			print("Executing a mission. Mission Index: " .. iIndex .. "    Player: " .. pPlayer:GetID() .. "  on game turn: " .. iGameTurn .. "  Planned Turn: " .. gAiMissionTable[iIndex][3] .. "    Target Civ: " .. gAiMissionTable[iIndex][7] .. "    Target City: " .. gAiMissionTable[iIndex][8]:GetName() .. "    Mission Number: " .. iMissionNumber .. "    Results: " .. gAiMissionTable[iIndex][9])

		end	
	end
end


-- =================================================================================
--			Process AI Covert Missions
--
--  Add function that drives AI CO missions - AI can use all missions except #1 (Conduct Recon)
-- 
-- =================================================================================

function PlayerDoTurn_ProcessAI_CovertOperations( iPlayer )									-- Only major AI civs can use covert missions.  Change to let city states launch?
	local pPlayer = Players[iPlayer]														-- Get pPlayer from player ID				
	local iGameTurn = Game.GetGameTurn()													-- Get current game turn
	local iCurrentEra = pPlayer:GetCurrentEra()												-- Get current player's Era level

	-- =========================================================================================================================
	-- Process AI Covert Operation Mission Components
	
	if (pPlayer:IsAlive()) and (not pPlayer:IsHuman()) and (not pPlayer:IsMinorCiv()) and (not pPlayer:IsBarbarian()) then	-- Process AI covert operations only for living major AI civs	
	
		Component1_Schedule_Mission(pPlayer, iGameTurn, iCurrentEra)						-- AI Process Component 1: Schedule an AI CO Mission:  To launch or not to launch, that is the question...		
		Component3_Execute_Mission(pPlayer, iGameTurn, iCurrentEra)							-- AI Process Component 3: Process an AI CO Mission
	
	elseif pPlayer:IsHuman() then
		Component2_Warn_Mission(pPlayer, iGameTurn)											-- AI Process Component 2: Warn human player only of an Upcoming AI CO Mission	
		--if iGameTurn % 5 == 0 then DisplayAIMissionTable(iGameTurn) end						-- Display the mission table contents
	end
end
GameEvents.PlayerDoTurn.Add( PlayerDoTurn_ProcessAI_CovertOperations )
