-- Covert Operations Popup Functions
-- Author: sman
-- DateCreated: 12/21/17 1:21:51 PM
--------------------------------------------------------------

--include ("00_Globals.lua")
--include ("50_Recon_Functions.lua")

include("PlotMouseoverInclude.lua")

-- ===============================================================================================
--		Globals


-- ===============================================================================================
--		Support Functions

function GetTargetCityInfo(pUnit, pPlot)													-- Determines if Operative is adjacent to enemy city.  Returns city instance
	local pReturnCity = nil																	-- Default return of nil means Operative is not adjacent to enemy city
	for loop, direction in ipairs(directions) do
		local adjPlot = Map.PlotDirection(pPlot:GetX(), pPlot:GetY(), direction)
		if adjPlot then
			if (adjPlot:IsCity()) then
				pAdjacentCity = adjPlot:GetPlotCity()
				if pAdjacentCity:GetOwner() ~= pUnit:GetOwner() then						-- City must be owned by a different civ
					--print("--> Not a friendly city.  ID: " ..  pAdjacentCity:GetID() .. "   Name: " .. pCity:GetName())
					pReturnCity = pAdjacentCity												-- Return city instance 					
				end
			end
		end
	end
	return pReturnCity
end

function GetTargetPlotInfo(pUnit, pPlot)													-- Determines if plot operatros is on has an improvement.  Returns improvement type and whether the plot is pillaged or not
	local PlotImprovement = pPlot:GetImprovementType()										-- PlotImprovement (return value); method returns -1 for no improvement
	local IsImprovementPillaged = NO														-- Default/assumption is improvements are not normally pillaged
	
	if pPlot:GetOwner() ~= pUnit:GetOwner() then											-- Plot must be owned by a different civ		
		if pPlot:IsImprovementPillaged() == true then										-- If improvement is pillaged, change default return value
			IsImprovementPillaged = YES
		end
		if not PlotImprovement then															-- Null value converted to -1, to indicate no improvements on this plot
			PlotImprovement = -1	
			IsImprovementPillaged = NO														-- Force plundered to NO if no improvement on plot
		end	
	else
		PlotImprovement = -1																-- Automatically return -1 if on friendly territory
		IsImprovementPillaged = NO															-- Automatically return NO for pillage status if on friendly territory
	end
	return PlotImprovement, IsImprovementPillaged
end


function SendNotification(iKey, NotificationStringLong, NotificationStringFading, displayUnit, x, y)		-- Displays notification for the right side of the screen, iKey parameter determines which notification displays
	--print ("inside SendNotify.  ikey = " .. iKey .. "  disp unit = " .. displayUnit:GetName())
	if iKey == 1 then																		-- Display unit icon in a circle
		Players[Game.GetActivePlayer()]:AddNotification(NotificationTypes.NOTIFICATION_GREAT_PERSON_ACTIVE_PLAYER, NotificationStringLong, NotificationStringFading,
			x, y, displayUnit:GetUnitType(), -1)

	elseif iKey == 2 then																	-- Display unit icon in animated blades/red bg
		Players[Game.GetActivePlayer()]:AddNotification(NotificationTypes.NOTIFICATION_UNIT_DIED, NotificationStringLong, NotificationStringFading,
			x, y, displayUnit:GetUnitType(), -1)

	elseif iKey == 3 then																	-- Diplay unit icon with crosshairs and red bg
		Players[Game.GetActivePlayer()]:AddNotification(NotificationTypes.NOTIFICATION_REBELS, NotificationStringLong, NotificationStringFading,
			x, y, displayUnit:GetUnitType(), -1)

	elseif iKey == 4 then																	-- Diplay blue spy icon: for AI Covert Ops Warnings; click on Notification icon brings camera to XY
		Players[Game.GetActivePlayer()]:AddNotification(NotificationTypes.NOTIFICATION_SPY_RIG_ELECTION_SUCCESS, NotificationStringLong, NotificationStringFading,
			x, y)

	elseif iKey == 5 then																	-- Diplay red spy icon: for successful AI Covert Ops; click on Notification icon brings camera to XY 
		Players[Game.GetActivePlayer()]:AddNotification(NotificationTypes.NOTIFICATION_SPY_WAS_KILLED, NotificationStringLong, NotificationStringFading,
			x, y)
	end
end


function GetRandomBuilding(targetCity)														-- Returns a random building type that exists in the city; no wonders returned!
	local randomBuildingType = -1															-- If no valid buildings exist, return nil
	local buildingsInCityTable = {}
	local buildingIDX = 0
	
	for row in GameInfo.Buildings() do														-- Scroll through all buildings in the Buildings table
		local buildnum =  targetCity:GetNumRealBuilding(row.ID)
		if targetCity:GetNumRealBuilding(row.ID) ~= 0  then									-- If the city has more than zero of these building types, then...
			local BClass = GameInfo.Buildings[row.ID].BuildingClass							-- Get the Building class of the building in the city
			local BWonder = GameInfo.BuildingClasses["" .. BClass .. ""].MaxGlobalInstances		-- Get MGI amount; normal buildings = 0; global wonders ~= 0
			local BNational = GameInfo.BuildingClasses["" .. BClass .. ""].MaxPlayerInstances	-- Get MPI amount; normal buildings = 0; national wonders ~= 0
			if (BWonder == -1) and (BNational == -1) then									-- If not a Global or National wonder then...
				buildingIDX = buildingIDX + 1												-- Increment the table index
				buildingsInCityTable[buildingIDX] = row.ID
			end
		end
	end
	if #buildingsInCityTable > 0  then														-- If there are one or more "normal" buildings in the city, then...
		local luckyBuilding = math.random(1, #buildingsInCityTable)							-- Generate random number between 1 and total count of normal buildings in city
		randomBuildingType = buildingsInCityTable[luckyBuilding]							-- Set return value to building ID
	end	
	return randomBuildingType																-- Return a existing normal building type or -1
end


function SpawnRebelUnits(targetCity, upperSpawnLimit, iSpawnCivID)							-- Spawns a number of Barb units; type is the most advanced unit that civ has; spawn civ is the playerID num of the units
	if upperSpawnLimit < 1 then upperSpawnLimit = 1 end										-- Minimum upper limit is 1					
	local spawnTotal = math.random(1, upperSpawnLimit)										-- Random number of Barb units to spawn; Ancient Era is 1..1, Infor Era is 1..5	
	local spawnUnitType = WARRIOR															-- Default partisan is a Barb Warrior		
	local targetCityPlot = Map.GetPlot(targetCity:GetX(), targetCity:GetY())
	local freePlots = {}

	for _, iUnit in pairs(LandInfantry) do													-- Find most tech advanced infantry unit available; break loop early when it is found
		if targetCity:CanTrain(iUnit) then													-- Can the target city train this unit?  if so - set it as the most advanced, the terminate search loop
			spawnUnitType = iUnit															-- Set the unit to spawn as the most advanced one
			break																			-- Terminate the search loop early
		end
	end
	
	for iDX = -3, 3 do																		-- Search all ploys near target city: within x & y  +/- 3 tiles
		for iDY = -3, 3 do	
			local pTargetPlot = Map.PlotXYWithRangeCheck(targetCity:GetX(), targetCity:GetY(), iDX, iDY, 3)		-- Get the plot to examine					
			if pTargetPlot then																-- If plot is valid, then...
				local plotNumUnits = pTargetPlot:GetNumUnits()										-- Get the number of units on this plot
				if not (pTargetPlot:IsWater()) and not (plotNumUnits>0) and (pTargetPlot ~= targetCity:Plot() and not (pTargetPlot:IsMountain()) ) then	-- Only include empty land plots
					table.insert(freePlots, pTargetPlot)									-- Insert this plot into the freePlots table
				end
			end
		end
	end	
			
	for i = 1, #freePlots, 1 do																-- Shuffle the plot table, so units will appear at random locations surrounding targetCity
		local k = math.random(1, #freePlots)												-- Iterate through table sequentially, 1 .. #freePlots
		freePlots[i], freePlots[k] = freePlots[k], freePlots[i]								-- Swap the iterated plot with a random plot anywhere else in the table
	end
			
	if #freePlots < spawnTotal then spawnTotal = #freePlots end								-- Ensure no more units than valid plots spawn
	
	for n, plot in pairs(freePlots) do														-- Loop through the sorted plot table 							
		if n > spawnTotal then break end													-- Once n is more than the number of units to spawn, exit the loop early
			local partisanUnit = Players[iSpawnCivID]:InitUnit(spawnUnitType, plot:GetX(), plot:GetY())	-- spawn the iSpawnCivID civs unit 'spawnUnitType' at just outside target city
			partisanUnit:SetMoves(0)														-- Spawned units cannot move the turn they are created		
	end
	return spawnTotal, spawnUnitType
end


function ClearMissionTable()																-- Clears ValidMissions table back to empty state
	g_ValidMissions = { [CONDUCT_RECON] = { NO, 0, 0, 0}, [POISON_WATER] = { NO, 0, 0, 0}, [SPREAD_RUMORS] = { NO, 0, 0, 0}, [PILLAGE_IMPROVEMENT] = { NO, 0, 0, 0}, 
						[SABOTAGE_PRODUCTION] = { NO, 0, 0, 0}, [STEAL_GOLD] = { NO, 0, 0, 0}, [SPREAD_PLAGUE] = { NO, 0, 0, 0}, [DESTROY_BUILDING] = { NO, 0, 0, 0}, 
						[RECRUIT_PARTISANS] = { NO, 0, 0, 0}, [SPREAD_PROPAGANDA] = { NO, 0, 0, 0}, [CAUSE_RESISTANCE] = { NO, 0, 0, 0}, [CAUSE_ANARCHY] = { NO, 0, 0, 0}, 
						[STEAL_CITY] = { NO, 0, 0, 0}, [ABORT_MISSION] = { YES, 0, 0, 0} }
end


function BuildValidMissionTable(pPlayer, pUnit, pPlot)										-- Table is built by determining which missions are valid for this unit, based on...
																							-- its location, etc.  It passes through once, making determination for all possible missions
	--print("Inside BVMT for player: " .. pPlayer:GetName() .. " and unit " .. pUnit:GetName() .. " located at " .. pPlot:GetX() .. " / " .. pPlot:GetY())		
	--  These variables hold data that is used to determine what missions are valid for a particular unit
	local pTargetCity  = GetTargetCityInfo(pUnit, pPlot)									-- Test for adjacent city; only returns pCity if adjcent city is enemy; returns nil if adjacent city is friendly
	local cityName = "no adjacent city"
	local IsTargetCityCapital = NO															-- Not Boolean, defaultl to NO
	local iTargetPlayerID = 0																-- Default to the active (human) player
	
	ClearMissionTable()																		-- Ensure Mission Table is defaulting to all missions off - must be enabled here

	-- These statements test certain conditions concerning the location of the Operative unit and what its surroundings are, i.e. city adjacent, enemy terrain, etc.	
	if pTargetCity then																		-- Tests for city adjacency
		iTargetPlayerID = pTargetCity:GetOwner()											-- Captures the Player ID of the city owner
		cityName = pTargetCity:GetName()													-- Returns city name		
		if pTargetCity:IsCapital() then														-- If adjacent city is an enemy capital, set that flag to YES
			IsTargetCityCapital = YES
		end	
	end

	local PlotImprovement, IsImprovementPillaged = GetTargetPlotInfo(pUnit, pPlot)			-- Test for improvement at unit location and its status; returns improvement type only if on enemy territory
	if iTargetPlayerID == 0 then															-- If true, then there is no pCity instance, need to get plot owner to fix this value
		iTargetPlayerID = pPlot:GetOwner()													-- Captures the Player ID of the plot owner
	end	

	if IsImprovementPillaged == YES then													-- If improvement is pillaged, ensure the improvement type is set to negative
		PlotImprovement = PlotImprovement * -1												--		version of the type - so function can prevent an attempt to pillage already damaged plot
	end
				
	-- pTargetCity - for these missions, the Operative Unit must be adjacent to enemy city, otherwise the mission is invalid; most missions require city adjacency
	if pTargetCity then	
		if iTargetPlayerID <= GameDefines.MAX_MAJOR_CIVS then								-- These missions can NOT be conducted against City States, only major civs
		--if iTargetPlayerID <+ GameDefines.MAX_MAJOR_CIVS then								-- Test this!

			g_ValidMissions[SPREAD_RUMORS][1] = YES											-- Mission: Spread Rumors (3) - Unit is adjacent to enemy city, mission is valid
			g_ValidMissions[SPREAD_RUMORS][2] = pTargetCity									-- Pointer to adjacent city instance
			g_ValidMissions[SPREAD_RUMORS][3] = IsTargetCityCapital							-- Not Boolean value; not applicable for this mission, but still tracked
			g_ValidMissions[SPREAD_RUMORS][4] = iTargetPlayerID								-- Target Player ID

			g_ValidMissions[STEAL_GOLD][1] = YES											-- Mission: Sabotage Production (6) - Unit is adjacent to enemy city, mission is valid
			g_ValidMissions[STEAL_GOLD][2] = pTargetCity									-- Pointer to adjacent city instance
			g_ValidMissions[STEAL_GOLD][3] = IsTargetCityCapital							-- Not Boolean value; applicable for this mission: more gold to steal if city is capital
			g_ValidMissions[STEAL_GOLD][4] = iTargetPlayerID								-- Target Player ID
		
			g_ValidMissions[SPREAD_PROPAGANDA][1] = YES										-- Mission: Spread Propaganda (10) - Unit is adjacent to enemy city, mission is valid; more powerful version of Spread Rumors
			g_ValidMissions[SPREAD_PROPAGANDA][2] = pTargetCity								-- Pointer to adjacent city instance
			g_ValidMissions[SPREAD_PROPAGANDA][3] = IsTargetCityCapital						-- Not Boolean value; not applicable for this mission, but still tracked
			g_ValidMissions[SPREAD_PROPAGANDA][4] = iTargetPlayerID							-- Target Player ID

			g_ValidMissions[CAUSE_RESISTANCE][1] = YES										-- Mission: Cause Anarachy (11) - Unit is adjacent to enemy city, mission is valid
			g_ValidMissions[CAUSE_RESISTANCE][2] = pTargetCity								-- Pointer to adjacent city instance
			g_ValidMissions[CAUSE_RESISTANCE][3] = IsTargetCityCapital						-- Not Boolean value; not applicable for this mission, but still tracked
			g_ValidMissions[CAUSE_RESISTANCE][4] = iTargetPlayerID							-- Target Player ID
				
			g_ValidMissions[CAUSE_ANARCHY][1] = YES											-- Mission: Incite City Revolt (12) - Unit is adjacent to enemy non-capital city, mission is valid
			g_ValidMissions[CAUSE_ANARCHY][2] = pTargetCity									-- Pointer to adjacent city instance
			g_ValidMissions[CAUSE_ANARCHY][3] = IsTargetCityCapital							-- Not Boolean value; not applicable for this mission after initial check, but still tracked
			g_ValidMissions[CAUSE_ANARCHY][4] = iTargetPlayerID								-- Target Player ID
		
			if IsTargetCityCapital ~= YES then												-- Cannot conduct these missions against the capital or city states
				g_ValidMissions[STEAL_CITY][1] = YES										-- Mission: Steal City (13) - Unit is adjacent to non-capital enemy city, mission is valid
				g_ValidMissions[STEAL_CITY][2] = pTargetCity								-- Pointer to adjacent city instance
				g_ValidMissions[STEAL_CITY][3] = IsTargetCityCapital						-- Not Boolean value; not applicable for this mission after initial check, but still tracked		g_ValidMissions[CONDUCT_RECON][4] = iTargetPlayerID									-- Target Player ID
				g_ValidMissions[STEAL_CITY][4] = iTargetPlayerID							-- Target Player ID
			end

		end																				-- These missions can be conducted against ALL civs, city states too	
			
		g_ValidMissions[CONDUCT_RECON][1] = YES											-- Mission: Conduct Recon (1) - Unit is adjacent to enemy city, mission is valid
		g_ValidMissions[CONDUCT_RECON][2] = pTargetCity									-- Pointer to adjacent city instance
		g_ValidMissions[CONDUCT_RECON][3] = IsTargetCityCapital							-- Not Boolean value; not applicable for this mission, but still tracked		
		g_ValidMissions[CONDUCT_RECON][4] = iTargetPlayerID								-- Target Player ID
	
		g_ValidMissions[POISON_WATER][1] = YES											-- Mission: Poison Water Supply (2) - Unit is adjacent to enemy city, mission is valid
		g_ValidMissions[POISON_WATER][2] = pTargetCity									-- Pointer to adjacent city instance
		g_ValidMissions[POISON_WATER][3] = IsTargetCityCapital							-- Not Boolean value; not applicable for this mission, but still tracked
		g_ValidMissions[POISON_WATER][4] = iTargetPlayerID								-- Target Player ID

		g_ValidMissions[SABOTAGE_PRODUCTION][1] = YES									-- Mission: Sabotage Production (5) - Unit is adjacent to enemy city, mission is valid
		g_ValidMissions[SABOTAGE_PRODUCTION][2] = pTargetCity							-- Pointer to adjacent city instance
		g_ValidMissions[SABOTAGE_PRODUCTION][3] = IsTargetCityCapital					-- Not Boolean value; not applicable for this mission, but still tracked
		g_ValidMissions[SABOTAGE_PRODUCTION][4] = iTargetPlayerID						-- Target Player ID

		g_ValidMissions[SPREAD_PLAGUE][1] = YES											-- Mission: Spread Plague (7) - Unit is adjacent to enemy city, mission is valid
		g_ValidMissions[SPREAD_PLAGUE][2] = pTargetCity									-- Pointer to adjacent city instance
		g_ValidMissions[SPREAD_PLAGUE][3] = IsTargetCityCapital							-- Not Boolean value; not applicable for this mission, but still tracked
		g_ValidMissions[SPREAD_PLAGUE][4] = iTargetPlayerID								-- Target Player ID

		g_ValidMissions[DESTROY_BUILDING][1] = YES										-- Mission: Destroy Building (8) - Unit is adjacent to enemy city, mission is valid
		g_ValidMissions[DESTROY_BUILDING][2] = pTargetCity								-- Pointer to adjacent city instance
		g_ValidMissions[DESTROY_BUILDING][3] = IsTargetCityCapital						-- Not Boolean value; not applicable for this mission, but still tracked
		g_ValidMissions[DESTROY_BUILDING][4] = iTargetPlayerID							-- Target Player ID

		g_ValidMissions[RECRUIT_PARTISANS][1] = YES										-- Mission: Recruit Partisans (9) - Unit is adjacent to enemy city, mission is valid
		g_ValidMissions[RECRUIT_PARTISANS][2] = pTargetCity								-- Pointer to adjacent city instance
		g_ValidMissions[RECRUIT_PARTISANS][3] = IsTargetCityCapital						-- Not Boolean value; not applicable for this mission, but still tracked
		g_ValidMissions[RECRUIT_PARTISANS][4] = iTargetPlayerID							-- Target Player ID
		
	end
	
	if PlotImprovement > -1 then															-- Mission: Pillage (4) - Unit must be on enemy terrain improvement
		g_ValidMissions[PILLAGE_IMPROVEMENT][1] = YES										-- Unit is on enemy plot with a non-pillaged improvement, mission is valid
		g_ValidMissions[PILLAGE_IMPROVEMENT][2] = PlotImprovement							-- Improvement Type of plot where Operative is located
		g_ValidMissions[PILLAGE_IMPROVEMENT][3] = IsImprovementPillaged						-- Is pillaged?  Value = YES || NO, not bool true || false
		g_ValidMissions[PILLAGE_IMPROVEMENT][4] = iTargetPlayerID									-- Target Player ID
	end
		
	g_ValidMissions[ABORT_MISSION] = { YES, 0, 0, 0}										-- Mission: Abort Mission (14) - Can always select the "Abort Mission" choice
end


function GetBuildingInfluence(pTargetCity)													-- Returns the total influence of certain buildings if the city has them
	-- Defensive buildings only influence the chance of mission success and operator survival; does not impact international reaction
	local iInfluenceCount = 0																-- Initialize tally at 0	
	for i = 1, #g_tBuildingList do															-- Cycle through all the building types in the tBuildingList
		if pTargetCity:GetNumBuilding(g_tBuildingList[i][1]) > 0 then						-- Does this city have this building?
			iInfluenceCount = iInfluenceCount + g_tBuildingList[i][2]						-- If so, bump the tally by the influence amount [2]
			--print("city has bld type: " .. g_tBuildingList[i][1] .. "  raising influence by " .. g_tBuildingList[i][2] .. "  total is now: " .. iInfluenceCount)
		end
	end	
	return iInfluenceCount																	-- Return the tally as the total influence of all appropriate buildings in city
end


function GetUnitInfluence(pTargetCity)														-- Returns the total influence of the units in the AO; some units help more than others
	local iCityOwnerID = pTargetCity:GetOwner()												-- Get the targe city owner's ID
	local iUnitInfluence = 0																-- Tally of units in City tiles: normal unit=1, Covert Operator=CO level*3
	for i = 0, pTargetCity:GetNumCityPlots() - 1, 1 do										-- Find open plots to spawn units on by examining all city owned plots
		local pWorkingPlot = pTargetCity:GetCityIndexPlot( i )								-- Find the pointer to the current plot (i)
		local iPlotUnitCount = pWorkingPlot:GetNumUnits()									-- Get the number of units on this plot
		    
		for i = 0, iPlotUnitCount - 1, 1 do													-- Cycle through all units in the plot
    		local pFoundUnit = pWorkingPlot:GetUnit(i)										-- Get unit(i) pointer
			if (pFoundUnit:GetOwner() == iCityOwnerID) then									-- If unit owner is the same as city owner (i.e. friendly unit), then				
				local iInfluenceAmount = 1													-- Default unit influence is 1
				local iCOLevel = GameInfo.Units[pFoundUnit:GetUnitType()].IsCovertOperatorUnit		-- Get the unit's Covert Operator level
				if iCOLevel > 0 then														-- If the level is 1 or more, the unit is a CO, then
					iInfluenceAmount = iCOLevel * 3											-- Unit's influence is now the CO level * 3
				end				
				iUnitInfluence = iUnitInfluence + iInfluenceAmount							-- Bump the tally by the unit's influence amount
			end
		end
	end	
	return iUnitInfluence
end


function GetRandomImprovement(pTargetCity)													-- Returns a Plot pointer if it finds a random improvement; nil otherwise
	local pReturnPlot = nil																	-- Return value is default nil
	local tImprovementTable = {}
	for i = 0, pTargetCity:GetNumCityPlots() - 1, 1 do										-- Find open plots to spawn units on by examining all city owned plots
		local pWorkingPlot = pTargetCity:GetCityIndexPlot( i )								-- Find the pointer to the current plot (i)
		
		if not pWorkingPlot:IsCity() then													-- Don't add city plot to ImprovementTable; check all others			
			if pWorkingPlot:GetImprovementType() ~= -1 then									-- GetImprovementType returns -1 for no improvement				
				if not pWorkingPlot:IsImprovementPillaged() then							-- If the plot is improved and it is not pillaged, then

					tImprovementTable[#tImprovementTable+1] = pWorkingPlot					-- Add the plot pointer to the table
				end
			end
		end
	end

	if #tImprovementTable > 0 then
		local iPlotIndex = math.random(1, #tImprovementTable)
		pReturnPlot = tImprovementTable[iPlotIndex]
	end
	return pReturnPlot
end


function GetRandomCity(pPlayer, bIsCapitalOK)												-- Returns a (semi) random city pointer in pPlayer's civ; if bIsCapitalOK = false, then return any non-capital city or nil if pPlayer only has 1 city
	-- Rules: 25% of the time, will return the capital, 25% of the time it will return the largest city (by population); 50% of the time a truly random city
	local iNumCities = pPlayer:GetNumCities() or 0											-- Track total number of pPlayer cities; default is 0	
	if iNumCities < 1 then																	-- If pPlayer has no cities (complete kills on?) return nil, leave early
		print("ERROR - player " .. pPlayer" .. has no cities!")
		return nil
	end
	local pCity = nil																		-- Can return nil if pPlayer has no cities or capital (complete kills on); should never happen from here, due to trap at top of function! 
	local iReturnCity = math.random(1, 100)													-- What kind of city do we return?
	
	if not bIsCapitalOK then iReturnCity = math.random(26, 100) end							-- If capital is NOT ok, then ensure either largest city or truly random city (largest city becomes more common by default: 33% chance instead of 25%)
	
	if iReturnCity < 25 then																-- Return the capital
		pCity = pPlayer:GetCapitalCity()													-- Set the return city pointer to the capital city pointer
		
	elseif iReturnCity < 50 then															-- Return the largest city, often the capital - which must be tested for if NOT bIsCapitalOK
		local iLargestPopulation = 0
		for iLoopCity in pPlayer:Cities() do												-- Cycle through all of pPlayer's cities
			if ( bIsCapitalOK ) or															-- If capitals are OK, then process all cities    OR
					( not bIsCapitalOK and not iLoopCity:IsCapital() ) then					-- If capitals are NOT OK and this city is not a capital, ok to process
				local iLoopCityPop = iLoopCity:GetPopulation()								-- Get the loop city pop
				if iLoopCityPop > iLargestPopulation then									-- If it's more than our previously largest city's pop, then
					iLargestPopulation = iLoopCityPop										-- Set the largest pop to loop city's pop
					pCity = iLoopCity														-- Set city pointer to loop city's pointed
				end
			end
		end																					-- At loop's end, we should have largest city's pointer; a nil should only happen if bIsCapitalOK is true and pPlayer only has 1 city	
	
	else																					-- Return any city in the civ
		local iCounter = 0																	-- Temp index
		local tAllowedCityTable = {}														-- Table of "allowed" cities to consider		
		for iLoopCity in pPlayer:Cities() do												-- Loop through all pPlayer's cities
			if ( bIsCapitalOK ) or															-- If capitals are OK, OR
					( not bIsCapitalOK and not iLoopCity:IsCapital() ) then					-- If capitals are NOT OK and this city is not a capital, then			
				iCounter = iCounter + 1														-- Increment the counter of loop iterations/total number of legal cities
				tAllowedCityTable[iCounter] = iLoopCity										-- Add this city pointer to the "allowed cities table"
			end
		end
		if #tAllowedCityTable >= 1 then														-- If one or more legal cities, then
			local iReturnCity = math.random(1, #tAllowedCityTable)							-- Random index from 1 .. Total num legal cities
			pCity = tAllowedCityTable[iReturnCity]											-- Set the return value to the city pointer in the corresponding table element
		end
	end
	return pCity																			-- Return the city pointer
end


function CalculateEspionagePosture(pPlayer)													-- Returns the Esp. Posture or 0
	local iEP = 0																			-- Default return value
	local pCapCity = pPlayer:GetCapitalCity()												-- Get the capital city for pPlayer 	
	iEP = GetBuildingInfluence(pCapCity) + (pPlayer:GetNumSpies() * 2)						-- EP = Building influence (0..68) + (Num spies * 2); expand on this later?!?!?!?
	return iEP
end